SystemOrganization addCategory: #'SqueakNxt-EtoysCompiler'!

!LegoNxtMorph methodsFor: '*squeaknxt-etoyscompiler' stamp: 'Richo 8/5/2011 17:18'!
detectMainMorph
	^ self! !

!LegoNxtMorph methodsFor: '*squeaknxt-etoyscompiler' stamp: 'Richo 3/28/2012 12:38'!
download: aNxcTranslatedProgram
	| errors |
	[errors := ('USB' match: self portName)
		ifTrue: [aNxcTranslatedProgram downloadUsingUsb]
		ifFalse: [aNxcTranslatedProgram downloadUsingBluetooth: nxt protocol deviceName]]
		on: Error do: [:err | 
			"An error happened when trying to download.
			Just in case, disconnect"
			^ self disconnect;
				inform: err description].
	errors notEmpty
		ifTrue: ["This should *never* happen, but if it does
			it would be good to see the error"
			PEUtilities
				showSyntaxError: (errors inject: '' into: [:subTotal :next | 
					subTotal, String tab, next asString, String cr])
				translatedCode: aNxcTranslatedProgram asStringWithLineNumbers]! !

!LegoNxtMorph methodsFor: '*squeaknxt-etoyscompiler' stamp: 'Richo 1/11/2011 08:08'!
playerTranslator
	^ NxtPlayerTranslator on: self! !

!NxtPluggableDeviceMorph methodsFor: '*squeaknxt-etoyscompiler' stamp: 'Richo 3/27/2012 08:38'!
detectMainMorph
	"Looks for the brick in which it is connected and returns it"
	| ports |
	ports := #(#port1 #port2 #port3 #port4 #portA #portB #portC ).
	World allMorphsDo: [:morph |
		((morph isKindOf: LegoNxtMorph)
		and: [ports anySatisfy: [:port | (morph nxt perform: port) = device port]])
			ifTrue: [^ morph]].
	^ self error: 'The device is not plugged to any brick'! !

!NxtPluggableDeviceMorph methodsFor: '*squeaknxt-etoyscompiler' stamp: 'Richo 8/11/2011 16:12'!
download: aNxcTranslatedProgram
	^ self detectMainMorph download: aNxcTranslatedProgram! !

!NxtPluggableDeviceMorph methodsFor: '*squeaknxt-etoyscompiler' stamp: 'Richo 8/5/2011 17:18'!
playerTranslator
	^ self detectMainMorph playerTranslator! !

RBProgramNodeVisitor subclass: #NxtPlayerTranslator
	instanceVariableNames: 'brick motors sensors getterSelectors setterSelectors scriptSelectors variables methods currentMorph programName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-EtoysCompiler'!

!NxtPlayerTranslator commentStamp: 'Richo 1/13/2011 09:51' prior: 0!
A NxtPlayerTranslator is responsible for transforming a bunch of Etoys scripts (which may be compiled in a set of different Player subclasses) into a subclass of NxtProgram which can be translated to nxc.

This usually requires:
* compiling all methods and variables into one single class (it adds a prefix representing the receiver to avoid name clashes).
* renaming the message selectors to match the ones supported by nxc.
* changing all receivers to self.

Apart from that, it also takes care of representing etoys multithreading using nxc tasks and flag variables to decide when the task should stop.

See #translateMainScript:of: as a starting point on how to use this class.

Instance Variables
	brick:		<LegoNxtMorph>
	motors:		<Array>
	sensors:		<Array>
	getterSelectors:		<Dictionary>
	setterSelectors:		<Dictionary>
	scriptSelectors:		<Dictionary>
	variables:		<Dictionary>
	methods:		<Set>
	currentMorph:		<Morph>
	programName:		<Symbol>


brick
	- An instance of LegoNxtMorph which contains the scripts we are going to translate.

motors
	- An array containing each motor connected to the nxt brick. Empty slots means nothing is connected to that port.

sensors
	- An array containing each sensor connected to the nxt brick. Empty slots means nothing is connected to that port.

getterSelectors
	- A dictionary containing all the slot getters. Each key is one of the morphs composing the robot, so we can know which slot corresponds to which morph.

setterSelectors
	- A dictionary containing all the slot setters. Each key is one of the morphs composing the robot, so we can know which slot corresponds to which morph.

scriptSelectors
	- A dictionary containing all the script selectors. Each key is one of the morphs composing the robot, so we can know which script corresponds to which morph.

variables
	- A dictionary containing all the instance variables (task flags included). Each key is one of the morphs composing the robot, so we can know which slot corresponds to which morph. Also, this dictionary stores the variable's type apart from its name.
	
methods
	- A set of strings containing the methods' source code to be compiled.

currentMorph
	- A pointer to one of the morphs composing the robot. Since all methods *must* be sent to one of these morphs (external references are not allowed), it defines which morph is currently represented by the 'self' variable. Very useful, but you shouldn't modify it directly. Instead use #withCurrentMorph:do:.

programName
	- The name of the temporary class we are using to compile the code.!

!NxtPlayerTranslator class methodsFor: 'utilities' stamp: 'Richo 3/21/2012 21:01'!
allPrimitiveRulesSelectors
	^ Array streamContents: [:stream || current |
		"Collect all selectors in the NXProgram protocol..."
		current := NXProgram.
		[stream nextPutAll: current selectors]
			doWhileTrue: [(current := current superclass) ~= CObject superclass].
		"... and in all the primitive types' protocol as well"
		CPrimitiveType withAllSubclassesDo: [:class |
			stream nextPutAll: class selectors]]! !

!NxtPlayerTranslator class methodsFor: 'utilities' stamp: 'Richo 7/29/2011 16:26'!
avoidNameClash
"
Utility method. Compiles an empty method in SqueakNxtPlayer for each nxc primitive rule selector. This way we prevent the user from creating a script with an invalid name.
Execute this if the list of rules changes."
"
self avoidNameClash.
"
self allPrimitiveRulesSelectors, #(main setUp) 
	select: [:each | (each isKeyword and: [each numArgs = 1]) or: [each isUnary]]
	thenDo: [:selector | | method |
	method := String streamContents: [:stream |
		selector isUnary
			ifTrue: [stream nextPutAll: selector]
			ifFalse: [selector keywords doWithIndex: [:each :index | stream nextPutAll: each; space; nextPutAll: 'obj', index asString; space]].
		stream cr; nextPutAll: '"This method''s name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"'
		].
	"Warning, we don't want to override an existing method"
	(SqueakNxtPlayer canUnderstand: selector)
		ifFalse: [SqueakNxtPlayer compile: method classified: #'*squeaknxt-etoyscompiler-nxc commands']
].! !

!NxtPlayerTranslator class methodsFor: 'instance creation' stamp: 'Richo 8/11/2011 17:21'!
on: aLegoNxtMorph
	^ self basicNew 
		initializeWith: aLegoNxtMorph;
		yourself! !

!NxtPlayerTranslator methodsFor: 'visiting' stamp: 'Richo 3/29/2012 15:00'!
acceptMessageNode: aMessageNode 
	| selector |
	selector := aMessageNode selector.
	"Check for invalid selector or invalid receiver"
	self assert: [self isValidSelector: selector];		
		assert: [self isValidReceiver: aMessageNode receiver].
	"Is it a rule?"
	(self rules includesKey: selector)
		ifTrue: [^(self rules at: selector)
				value: aMessageNode].
	"Is it an etoys message?"
	(self etoysMessages includesKey: selector)
		ifTrue: [(self etoysMessages at: selector)
				value: aMessageNode].
	"Is it a getter?"
	(self allGetterSelectors includes: selector)
		ifTrue: [| newNode | 
			self
				withCurrentMorph: (self morphFor: aMessageNode receiver name)
				do: [newNode := self variable: (self slotForGetter: selector)].
			aMessageNode
				become: (newNode parent: aMessageNode parent).
			^ self visitNode: aMessageNode].
	"Is it a setter?"
	(self allSetterSelectors includes: selector)
		ifTrue: [| newNode | 
			self
				withCurrentMorph: (self morphFor: aMessageNode receiver name)
				do: [newNode := self assignment: (self variable: (self slotForSetter: selector))
										to: aMessageNode arguments first].
			aMessageNode
				become: (newNode parent: aMessageNode parent).
			^ self visitNode: aMessageNode].
	"Is it a script call?"
	(self allScriptSelectors includes: selector)
		ifTrue: [self
				withCurrentMorph: (self morphFor: aMessageNode receiver name)
				do: ["Check for recursive call"
					self assert: [self isNotRecursiveCall: aMessageNode].
					aMessageNode
						selector: (self asLegalName: aMessageNode selector);
						
						receiver: (self variable: 'self')]].
	"We still need to visit the receiver and the arguments"
	^ super acceptMessageNode: aMessageNode! !

!NxtPlayerTranslator methodsFor: 'visiting' stamp: 'Richo 7/29/2011 16:04'!
acceptMethodNode: aMethodNode 
	"It goes over the method's arguments adding the #argTypes: pragma if necessary and modifies the selector to have a legal name"
	aMethodNode allArgumentVariables size = 1 
		ifTrue: [| varType | 
			varType := self asNxcType: (self currentPlayerClass scripts at: aMethodNode selector) argumentVariables first variableType.
			aMethodNode pragmas: (RBParser parseMethod: 'm <argTypes: #(' , varType , ')>') pragmas].
	aMethodNode
		selector: (self asLegalName: aMethodNode selector).
	^ super acceptMethodNode: aMethodNode! !

!NxtPlayerTranslator methodsFor: 'visiting' stamp: 'Richo 1/27/2011 12:50'!
acceptSequenceNode: aSequenceNode
	"If one of the statements has an invalid tile it will ignore it entirely. Only assertion failures will be ignored, other errors will be signaled, though"
	aSequenceNode statements do: [:each | [self visitNode: each] on: AssertionFailure do: [self becomeNoOp: each]]! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/10/2011 17:36'!
allGetterSelectors
	^ getterSelectors inject: #() into: [:subTotal :next | subTotal, next]! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/11/2011 08:32'!
allMorphs
	"Returns a collection with all the morphs composing the robot (it could contain nils)"
	^ {brick}, motors, sensors! !

!NxtPlayerTranslator methodsFor: 'iterating' stamp: 'Richo 1/11/2011 08:32'!
allMorphsDo: aBlock
	"Iterates over all morphs composing the robot taking care of posible nil occurrences"
	self allMorphs do: [:each |
		each notNil ifTrue: [aBlock value: each]
	]! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/10/2011 17:36'!
allScriptSelectors
	^ scriptSelectors inject: #() into: [:subTotal :next | subTotal, next]! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/10/2011 17:36'!
allSetterSelectors
	^ setterSelectors inject: #() into: [:subTotal :next | subTotal, next]! !

!NxtPlayerTranslator methodsFor: 'converting' stamp: 'Richo 1/10/2011 18:29'!
asLegalName: aString
"All variables, method names and task names must begin with a prefix representing the receiver."
^ currentMorph assuredPlayer uniqueNameForReference asLegalSelector, aString capitalized! !

!NxtPlayerTranslator methodsFor: 'converting' stamp: 'Richo 3/29/2012 14:31'!
asLegalScriptFlag: aSymbol 
	"Returns the name of the flag variable used to stop this script"
	^ self asLegalName: (self asScriptFlag: aSymbol)! !

!NxtPlayerTranslator methodsFor: 'converting' stamp: 'Richo 3/29/2012 14:31'!
asLegalScriptLastTime: aByteSymbol 
	^ self asLegalName: (self asScriptLastTime: aByteSymbol)! !

!NxtPlayerTranslator methodsFor: 'converting' stamp: 'Richo 7/29/2011 16:04'!
asNxcType: aSymbol
	"If the type is not valid just return the symbol. Hack to allow #firstTick to have #u32 type"
	^ aSymbol caseOf: {
		[#Number] -> [#CLong].
		[#Boolean] -> [#CBoolean].
		[#String] -> [#CString]
		} otherwise: [aSymbol]! !

!NxtPlayerTranslator methodsFor: 'converting' stamp: 'Richo 3/29/2012 14:31'!
asScriptFlag: aSymbol
	"Returns the name of the flag variable used to stop this script"
	^ (self asScriptName: aSymbol), 'Stepping'! !

!NxtPlayerTranslator methodsFor: 'converting' stamp: 'Richo 3/29/2012 14:31'!
asScriptLastTime: aSymbol
	^ aSymbol , 'LastTime'! !

!NxtPlayerTranslator methodsFor: 'converting' stamp: 'Richo 3/29/2012 14:31'!
asScriptName: aSymbol
	"Returns the name of this script"
	^ aSymbol, 'Script'! !

!NxtPlayerTranslator methodsFor: 'creating parse nodes' stamp: 'Richo 3/29/2012 15:00'!
assignment: aRBVariableNode to: aRBLiteralValueNode 
	^ RBAssignmentNode
		variable: aRBVariableNode
		value: aRBLiteralValueNode! !

!NxtPlayerTranslator methodsFor: 'converting' stamp: 'Richo 1/27/2011 18:58'!
becomeNoOp: aParseNode
	"Transforms the argument in an empty statement:
	
	self nxc: ''.
	"
	| noOp |
	noOp := RBMessageNode receiver: (self variable: 'self') selector: #nxc: arguments: {self literal: ''}.
	aParseNode become: (noOp parent: aParseNode parent)! !

!NxtPlayerTranslator methodsFor: 'accessing-rules' stamp: 'Richo 2/1/2011 11:30'!
buttonRules
| selfVar |
selfVar := self variable: 'self'.
^ {
#getIsPressedEnter -> [:node | 
	node arguments: {self literal: 3};
		 selector: #buttonPressed:;
		receiver: selfVar.
	super acceptMessageNode: node].

#getIsPressedLeft -> [:node | 
	node arguments: {self literal: 2};
		 selector: #buttonPressed:;
		receiver: selfVar.
	super acceptMessageNode: node].

#getIsPressedRight -> [:node | 
	node arguments: {self literal: 1};
		 selector: #buttonPressed:;
		receiver: selfVar.
	super acceptMessageNode: node]
}! !

!NxtPlayerTranslator methodsFor: 'initializing' stamp: 'Richo 10/21/2011 16:31'!
collectGetterSelectors
	"Collects all selectors corresponding to a slot getter in a dictionary whose keys are the morphs owning the slot"
	getterSelectors := Dictionary new.
	self allMorphsDo: [:morph | | selectors |
		selectors := getterSelectors at: morph ifAbsentPut: [OrderedCollection new].
		selectors addAll: (morph assuredPlayer slotInfo keys collect: [:selector | Utilities getterSelectorFor: selector])].
! !

!NxtPlayerTranslator methodsFor: 'initializing' stamp: 'Richo 1/26/2011 18:36'!
collectMotorsAndSensors
	"Finds out which motor and sensor morphs are connected to the nxt brick.
	It's not the cleanest way of doing it but it works..."
	| nxt |
	nxt := brick nxt.
	motors := Array new: 3.
	sensors := Array new: 4.
	World allMorphsDo: [:m |
		(m isKindOf: NxtPluggableDeviceMorph)
		ifTrue: [
			m device port caseOf: {
				[nxt portA] -> [motors at: 1 put: m].
				[nxt portB] -> [motors at: 2 put: m].
				[nxt portC] -> [motors at: 3 put: m].
				[nxt port1] -> [sensors at: 1 put: m].
				[nxt port2] -> [sensors at: 2 put: m].
				[nxt port3] -> [sensors at: 3 put: m].
				[nxt port4] -> [sensors at: 4 put: m].
			} otherwise: []
		]
	].! !

!NxtPlayerTranslator methodsFor: 'initializing' stamp: 'Richo 1/11/2011 08:10'!
collectRelevantInformation
	self collectMotorsAndSensors; 
		 collectSlots;
		 collectGetterSelectors;
		 collectSetterSelectors;
		 collectScriptSelectors! !

!NxtPlayerTranslator methodsFor: 'initializing' stamp: 'Richo 10/21/2011 16:31'!
collectScriptSelectors
	"Collects all selectors corresponding to a script in a dictionary whose keys are the morphs owning the methods"
	scriptSelectors := Dictionary new.
	self
		allMorphsDo: [:morph | 
			| selectors | 
			selectors := scriptSelectors
						at: morph
						ifAbsentPut: [OrderedCollection new].
			selectors
				addAll: (morph assuredPlayer class selectors
						reject: [:selector | ((getterSelectors at: morph)
									includes: selector)
								or: [(setterSelectors at: morph)
										includes: selector]])] ! !

!NxtPlayerTranslator methodsFor: 'initializing' stamp: 'Richo 10/21/2011 16:31'!
collectSetterSelectors
	"Collects all selectors corresponding to a slot setter in a dictionary whose keys are the morphs owning the slot"
	setterSelectors := Dictionary new.
	self allMorphsDo: [:morph | | selectors |
		selectors := setterSelectors at: morph ifAbsentPut: [OrderedCollection new].
		selectors addAll: (morph assuredPlayer slotInfo keys collect: [:selector | Utilities setterSelectorFor: selector])].
! !

!NxtPlayerTranslator methodsFor: 'initializing' stamp: 'Richo 1/26/2011 18:54'!
collectSlots
	"Adds all the slots and they respective types to the variables dictionary."
	variables := Dictionary new.
	self allMorphsDo: [:morph | | slots |
		slots := variables at: morph ifAbsent: [variables at: morph put: Dictionary new].
		morph assuredPlayer slotInfo keysAndValuesDo: [:slotName :slotInfo | slots at: slotName put: (slotInfo type -> (morph assuredPlayer instVarNamed: slotName))]]! !

!NxtPlayerTranslator methodsFor: 'translating' stamp: 'Richo 3/29/2012 18:35'!
compileLoopMethod
	methods add: (String streamContents: [:stream |
		stream nextPutAll: 'loop
	| now |
	<tempTypes: #(CUnsignedLong)>
	now := self currentTick.
	'.
	self scriptSelectorsDo: [:selector || code |
		code := '({1} and: [now - {2} > {3}])
		ifTrue: [{4} timesRepeat: [self {5}].
			{2} := now].' 
			format: {
				(self asLegalScriptFlag: selector).
				(self asLegalScriptLastTime: selector).
				(self delayOfScript: selector).
				(self frequencyOfScript: selector).
				(self asLegalName: selector)}.
		stream nextPutAll: code.
		]
	])! !

!NxtPlayerTranslator methodsFor: 'compiling' stamp: 'Richo 7/29/2011 16:39'!
compileNxtProgramSubclass
	| instVarNames instVarTypes |
	instVarNames := ''.
	instVarTypes := #().
	"First collect all variables"
	self
		variablesDo: [:varName :varTypeAndValue | 
			instVarNames := instVarNames, (self asLegalName: varName) , ' '.
			instVarTypes := instVarTypes, {self asNxcType: varTypeAndValue key}].
	"Then create the class"
	self createNxtProgramSubclassInstVarNames: instVarNames
		 instVarTypes: instVarTypes.
	"Finally compile all methods"
	methods
		do: [:each | self programClass compileSilently: each classified: #autogenerated]! !

!NxtPlayerTranslator methodsFor: 'translating' stamp: 'Richo 3/29/2012 14:45'!
compileSetUpMethod
"Compiles the #setup method, which gets executed once at the beginning of the program and its responsible for configuring the sensors and initialize all the inst vars. It also waits for 500 milliseconds (just to be sure all the sensors are correctly configured)"
| methodSource |
methodSource := WriteStream on: ''.
methodSource nextPutAll: 'setup'; cr.
"Configure sensors"
sensors doWithIndex: [:m :index |
	m isNil ifFalse: [ | zeroBasedIndex |
		zeroBasedIndex := (index - 1) asString.
		methodSource nextPutAll: '	self setSensor'.
		m device type caseOf: {
			[SensorType lowspeed9V] -> [methodSource nextPutAll: 'LowSpeed: ', zeroBasedIndex].
			[SensorType lightActive] -> [methodSource nextPutAll: 'Light: ', zeroBasedIndex, ' active: true'].
			[SensorType lightInactive] -> [methodSource nextPutAll: 'Light: ', zeroBasedIndex, ' active: false'].
			[SensorType soundDba] -> [methodSource nextPutAll: 'Sound: ', zeroBasedIndex, ' scaling: false'].
			[SensorType soundDb] -> [methodSource nextPutAll: 'Sound: ', zeroBasedIndex, ' scaling: true'].
			[SensorType switch] -> [methodSource nextPutAll: 'Touch: ', zeroBasedIndex]
		} otherwise: [].
		methodSource nextPut: $.; cr.
	]
].
"Wait"
methodSource 
	nextPutAll: 'self wait: 500.'; cr.
"Initialize variables"
self variablesDo: [:varName :varTypeAndValue | | value |
	value := varTypeAndValue value.
	value := (value isKindOf: RBProgramNode) 
				ifTrue: [value formattedCode]  "Is a message send"
				ifFalse: [value printString]. "Is a literal"
	methodSource 
		nextPutAll: (self asLegalName: varName), ' := ' , value , '.';
		cr].
methods add: methodSource contents! !

!NxtPlayerTranslator methodsFor: 'compiling' stamp: 'Richo 7/29/2011 17:35'!
createNxtProgramSubclassInstVarNames: aString instVarTypes: anArray 
	SystemChangeNotifier uniqueInstance doSilently: [
		NXProgram subclass: self programName
			instanceVariableNames: aString
			instanceVariableTypes: anArray
			classVariableNames: ''
			poolDictionaries: ''
			category: 'NxtPrograms']! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/10/2011 17:22'!
currentPlayer
	^ currentMorph assuredPlayer! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/10/2011 17:22'!
currentPlayerClass
	^ self currentPlayer class! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/27/2011 10:03'!
delayOfScript: scriptName
 	"To be in accordance with the script tick rate, we put a delay at the end of each task. How long the delay would be?"
	^ (1000 / (self currentPlayer scriptInstantiationForSelector: scriptName) tickingRate) rounded asString! !

!NxtPlayerTranslator methodsFor: 'accessing-rules' stamp: 'Richo 2/1/2011 11:27'!
displayRules
| selfVar installVariables displayX displayY |
displayX := (#displayX , self hashString) asSymbol.
displayY := (#displayY, self hashString) asSymbol.
installVariables := [:morph | 
	"I need to install both variables if any of these methods is founded (except for #clearScreen and #resetScreen)"
	(variables at: morph) at: displayX put: #Number -> (self currentPlayer getDisplayX).
	(variables at: morph) at: displayY put: #Number -> (self currentPlayer getDisplayY).].
selfVar := self variable: 'self'.
^ {
#setDisplayX: -> [:node | | newNode |
	self withCurrentMorph: (self morphFor: node receiver name) do: [
		installVariables value: currentMorph.
		newNode := RBAssignmentNode variable: (self variable: (self asLegalName: displayX)) value: node arguments first.
		node become: (newNode parent: node parent)].
	self visitNode: node].

#getDisplayX -> [:node | | newNode |
	self withCurrentMorph: (self morphFor: node receiver name) do: [
		installVariables value: currentMorph.
		newNode := self variable: (self asLegalName: displayX).
		node become: (newNode parent: node parent)
	].
	self visitNode: node].

#setDisplayY: -> [:node | | newNode |
	self withCurrentMorph: (self morphFor: node receiver name) do: [
		installVariables value: currentMorph.
		newNode := RBAssignmentNode variable: (self variable: (self asLegalName: displayY)) value: node arguments first.
		node become: (newNode parent: node parent)].
	self visitNode: node].

#getDisplayY -> [:node | | newNode |
	self withCurrentMorph: (self morphFor: node receiver name) do: [
		installVariables value: currentMorph.
		newNode := self variable: (self asLegalName: displayY).
		node become: (newNode parent: node parent)
	].
	self visitNode: node].

#displayNumber: -> [:node |
	self withCurrentMorph: (self morphFor: node receiver name) do: [
		installVariables value: currentMorph.
		node 
			receiver: selfVar; 
			arguments: node arguments , 
						{self variable: (self asLegalName: displayX). 
						self variable: (self asLegalName: displayY)}; 
			selector: #displayNumber:x:y:].
	super acceptMessageNode: node].

#displayText: -> [:node |
	self withCurrentMorph: (self morphFor: node receiver name) do: [
		installVariables value: currentMorph.
		node
			receiver: selfVar; 
			arguments: node arguments , 
						{self variable: (self asLegalName: displayX). 
						self variable: (self asLegalName: displayY)}; 
			selector: #displayText:x:y:].
	super acceptMessageNode: node].

#clearScreen -> [:node | node receiver: selfVar].

#resetScreen -> [:node | node receiver: selfVar]
}! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/27/2011 11:26'!
etoysMessages
^ {
"#grouped actually doesn't do anything so we just take it out"
#grouped -> [:node | node become: (node receiver parent: node parent)].

"I don't know why but recent versions of Etoys use this weird messages instead of the comparison operators:"
#eToysLT: -> [:node | node selector: #<].
#eToysLE: -> [:node | node selector: #<=].
#eToysEQ: -> [:node | node selector: #=].
#eToysNE: -> [:node | node selector: #~=].
#eToysGT: -> [:node | node selector: #>].
#eToysGE: -> [:node | node selector: #>=].
} as: Dictionary! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/27/2011 10:03'!
frequencyOfScript: scriptName
	"To be in accordance with the script frequency property we enclose the script call in a #timesRepeat: message. How many times should we repeat the script?"
	^ (self currentPlayer scriptInstantiationForSelector: scriptName) frequency rounded asString! !

!NxtPlayerTranslator methodsFor: 'hashing' stamp: 'Richo 2/1/2011 11:05'!
hashString
	^ self hash asString! !

!NxtPlayerTranslator methodsFor: 'initializing' stamp: 'Richo 8/11/2011 17:22'!
initializeWith: aLegoNxtMorph
	"By default we set the brick as the currentMorph"
	brick := currentMorph := aLegoNxtMorph.
	self initialize! !

!NxtPlayerTranslator methodsFor: 'testing' stamp: 'Richo 1/12/2011 14:50'!
isNotRecursiveCall: aMessageNode
	^ (self asLegalName: aMessageNode selector) ~= aMessageNode methodNode selector! !

!NxtPlayerTranslator methodsFor: 'testing' stamp: 'Richo 3/27/2012 10:15'!
isValidReceiver: aParseNode 
	"If the parse node is a variable it *must* be one of the valid morphs or an argument variable. I'm not sure about this, actually so we might have to test it more"
	| res |
	aParseNode isVariable
		ifTrue: [res := (aParseNode methodNode allArgumentVariables
							anySatisfy: [:each | each = aParseNode name])
						or: [(self morphFor: aParseNode name) notNil].
			self flag: #Richo.
			^ res].
	^ true! !

!NxtPlayerTranslator methodsFor: 'testing' stamp: 'Richo 7/29/2011 16:36'!
isValidSelector: selector 
	"Checks if the selector is one of the following:
	* A nxc primitive rule.
	* A player translator rule
	* An etoys message.
	* A slot accessor.
	* A script selector."
	^ self class allPrimitiveRulesSelectors asSet , self rules keys , self etoysMessages keys , self allGetterSelectors , self allSetterSelectors , self allScriptSelectors includes: selector! !

!NxtPlayerTranslator methodsFor: 'creating parse nodes' stamp: 'Richo 1/10/2011 17:33'!
literal: aLiteralObject
	^ RBLiteralValueNode value: aLiteralObject! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/27/2011 10:04'!
morphFor: receiverName
	"Returns the morph represented by the argument or nil if it's not a valid morph"
	^ receiverName = 'self'
		ifTrue: [currentMorph]
		ifFalse: [self allMorphs
				detect: [:each | each notNil and: [each assuredPlayer uniqueNameForReference = receiverName]] ifNone: [nil]]! !

!NxtPlayerTranslator methodsFor: 'accessing-rules' stamp: 'Richo 2/1/2011 11:30'!
motorRules
| selfVar |
selfVar := self variable: 'self'.
^ {
#setPower: -> [:node | 
	node arguments: {self nxcIndexOf: node receiver in: motors} , node arguments;
		selector: #onFwd:speed:;
		receiver: selfVar.
	super acceptMessageNode: node].

#float -> [:node |
	node arguments: {self nxcIndexOf: node receiver in: motors};
		selector: #float:;
		receiver: selfVar.
	super acceptMessageNode: node].

#brake -> [:node |
	node arguments: {self nxcIndexOf: node receiver in: motors};
		selector: #off:;
		receiver: selfVar.
	super acceptMessageNode: node].

#getPower -> [:node |
	node arguments: {self nxcIndexOf: node receiver in: motors};
		selector: #motorPower:;
		receiver: selfVar.
	super acceptMessageNode: node]
}! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 3/27/2012 12:56'!
nxcIndexOf: aVariableNode in: aCollection
	"Basically, it tries to determine if the variable node is representing an object in the collection. If it is, it returns its index as a literal node (zero based for nxc compatibility)"
	| index |
	aVariableNode name = 'self' 
		ifTrue: [(index := aCollection indexOf: currentMorph) = 0
			ifTrue: [index := nil]]
		ifFalse: [aCollection doWithIndex: [:morph :i| 
(morph notNil and: [morph assuredPlayer uniqueNameForReference = aVariableNode name]) ifTrue: [index := i. thisContext return: nil]]].
	"^ index isNil ifTrue: [self error: 'Index not found'] ifFalse: [self literal: index - 1]"
	self assert: index notNil.
	^ self literal: index - 1! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/10/2011 18:54'!
programClass
	"Returns the temporary class we are using to compile the code"
	^ Smalltalk at: self programName ifAbsent: []! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 8/5/2011 16:47'!
programName
	^ #NxtEtoysProgram! !

!NxtPlayerTranslator methodsFor: 'compiling' stamp: 'Richo 7/29/2011 17:29'!
removeNxtProgramSubclass
	"After translation is done we silently remove the temporary class"
	SystemChangeNotifier uniqueInstance doSilently: [
		self programClass ifNotNilDo: [:class | class removeFromSystem]]! !

!NxtPlayerTranslator methodsFor: 'accessing-rules' stamp: 'Richo 1/31/2011 13:12'!
rules
"These are the rules to modify the parse tree of the SqueakNxtPlayer scripts. Most of the time, I just need to rename the selector, change the receiver to self and set the correct list of arguments. Other times (such as with #getAudible or #getActive) the rule is a little more complicated"
^ (
	self motorRules, 
	self sensorRules, 
	self displayRules, 
	self scriptingRules, 
	self soundRules, 
	self buttonRules,
	self timerRules
	) as: Dictionary! !

!NxtPlayerTranslator methodsFor: 'iterating' stamp: 'Richo 1/11/2011 08:55'!
scriptSelectorsDo: aBlock 
	"Iterates over the scriptSelectors dictionary taking care of setting the current morph"
	scriptSelectors
		keysAndValuesDo: [:morph :selectors | self
				withCurrentMorph: morph
				do: [selectors do: aBlock]] ! !

!NxtPlayerTranslator methodsFor: 'accessing-rules' stamp: 'Richo 3/29/2012 15:41'!
scriptingRules
"For the tiles that receive a script name as a parameter, it first checks that the script actually belongs to the current morph, and if it doesn't then it just turns the node into a noop"
^ {
#startScript: -> [:node |
	self withCurrentMorph: (self morphFor: node receiver name) do: [| scriptName newNode |
		scriptName := node arguments first value.
		self assert: ((scriptSelectors at: currentMorph) includes: scriptName).
		newNode := self assignment: (self variable: (self asLegalScriptFlag: scriptName))
							to: (self literal: true).
		node become: (newNode parent: node parent)]].

#stopScript: -> [:node |
	self withCurrentMorph: (self morphFor: node receiver name) do: [| scriptName newNode |
		scriptName := node arguments first value.
		self assert: ((scriptSelectors at: currentMorph) includes: scriptName).
		newNode := self assignment: (self variable: (self asLegalScriptFlag: scriptName))
							to: (self literal: false).
		node become: (newNode parent: node parent)]].

#pauseScript: -> [:node | "Do the same as #stopScript:"
	(self rules at: #stopScript:) value: node].

#doScript: -> [:node | | args | args := node arguments. "This is the same as calling the script directly"
	node arguments: #(); selector: args first value.
	self visitNode: node "We need to visit the node again to fix the selector and the receiver"]
}! !

!NxtPlayerTranslator methodsFor: 'accessing-rules' stamp: 'Richo 3/27/2012 10:54'!
sensorRules
| selfVar |
selfVar := self variable: 'self'.
^ {
#getIsPressed -> [:node | 
	node arguments: {self nxcIndexOf: node receiver in: sensors};
		selector: #sensorBoolean:;
		receiver: selfVar.
	super acceptMessageNode: node].

#getAudible -> [:node | | copy | "(self sensorType: n) = 8"
	copy := node copy.
	node receiver: copy; 
		arguments: {self literal: SensorType soundDba}; 
		selector: #=.
	copy arguments: {self nxcIndexOf: copy receiver in: sensors}; 
		selector: #sensorType:; 
		receiver: selfVar.
	super acceptMessageNode: node].

#getActive -> [:node | | copy | "(self sensorType: n) = 5"
	copy := node copy.
	node receiver: copy; 
		arguments: {self literal: SensorType lightActive}; 
		selector: #=.
	copy arguments: {self nxcIndexOf: copy receiver in: sensors}; 
		selector: #sensorType:; 
		receiver: selfVar.
	super acceptMessageNode: node].

#getDistanceCm -> [:node |
	node arguments: {self nxcIndexOf: node receiver in: sensors};
		 selector: #sensorUS:;
		receiver: selfVar.
	super acceptMessageNode: node].

#getValue -> [:node |
	node arguments: {self nxcIndexOf: node receiver in: sensors};
			selector: #sensorValue:;
			receiver: selfVar.
	super acceptMessageNode: node].

#setActive: -> [:node |
	node arguments: {self nxcIndexOf: node receiver in: sensors}, node arguments;
		 selector: #setSensorLight:active:;
		receiver: selfVar.
	super acceptMessageNode: node].

#setAudible: -> [:node |
	node arguments: {self nxcIndexOf: node receiver in: sensors. 
						RBMessageNode receiver: node arguments first selector: #not};
		 selector: #setSensorSound:scaling:;
		receiver: selfVar.
	super acceptMessageNode: node].

#getRawValue -> [:node || morph selector |
	"If the receiver is an ultrasonic sensor, we need to 
	send #sensorUS: to get a correct value"
	morph := self morphFor: node receiver name.
	self assert: (sensors includes: morph).
	morph device type = SensorType lowspeed9V
		ifTrue: [selector := #sensorUS:]
		ifFalse: [selector := #sensorRaw:].
	node arguments: {self nxcIndexOf: node receiver in: sensors};
			selector: selector;
			receiver: selfVar.
	super acceptMessageNode: node].

#getNormalizedValue -> [:node |
	node arguments: {self nxcIndexOf: node receiver in: sensors};
			selector: #sensorNormalized:;
			receiver: selfVar.
	super acceptMessageNode: node].

#getScaledValue -> [:node |
	node arguments: {self nxcIndexOf: node receiver in: sensors};
			selector: #sensorScaled:;
			receiver: selfVar.
	super acceptMessageNode: node].
}! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 3/28/2012 15:00'!
slotForGetter: getterSelector
	"Returns the slot accessed by these selector"
	| slotName |
	slotName := currentMorph assuredPlayer slotInfo keys
				detect: [:slot | (Utilities getterSelectorFor: slot)
						= getterSelector]
				ifNone: [nil].
	self assert: slotName notNil.
	^ self asLegalName: slotName! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 3/28/2012 15:03'!
slotForSetter: setterSelector
	"Returns the slot accessed by these selector"
	| slotName |
	slotName := currentMorph assuredPlayer slotInfo keys
				detect: [:slot | (Utilities setterSelectorFor: slot)
						= setterSelector]
				ifNone: [nil].
	self assert: slotName notNil.
	^ self asLegalName: slotName! !

!NxtPlayerTranslator methodsFor: 'accessing-rules' stamp: 'Richo 2/1/2011 11:34'!
soundRules
| selfVar installVariables soundVolume soundDuration |
soundVolume := (#soundVolume, self hashString) asSymbol.
soundDuration := (#soundDuration, self hashString) asSymbol.
installVariables := [:morph | "I need to install both variables if any of these methods is founded (except for #stopPlaying)"
	(variables at: morph) at: soundVolume put: #Number -> (self currentPlayer getVolume).
	(variables at: morph) at: soundDuration put: #Number -> (self currentPlayer getDuration).].
selfVar := self variable: 'self'.
^ {
#setVolume: -> [:node | | newNode |
	self withCurrentMorph: (self morphFor: node receiver name) do: [
		installVariables value: currentMorph.
		newNode := RBAssignmentNode 
							variable: (self variable: (self asLegalName: soundVolume)) 
							value: node arguments first.
		node become: (newNode parent: node parent)].
	self visitNode: node].

#getVolume -> [:node | | newNode |
	self withCurrentMorph: (self morphFor: node receiver name) do: [
		installVariables value: currentMorph.
		newNode := self variable: (self asLegalName: soundVolume).
		node become: (newNode parent: node parent)].
	self visitNode: node].

#setDuration: -> [:node | | newNode |
	self withCurrentMorph: (self morphFor: node receiver name) do: [
		installVariables value: currentMorph.
		newNode := RBAssignmentNode 
							variable: (self variable: (self asLegalName: soundDuration)) 
							value: node arguments first.
		node become: (newNode parent: node parent)].
	self visitNode: node].

#getDuration -> [:node | | newNode |
	self withCurrentMorph: (self morphFor: node receiver name) do: [
		installVariables value: currentMorph.
		newNode := self variable: (self asLegalName: soundDuration).
		node become: (newNode parent: node parent)].
	self visitNode: node].

#playTone: -> [:node | 
	self withCurrentMorph: (self morphFor: node receiver name) do: [
		installVariables value: currentMorph.
		node receiver: selfVar;
		  arguments: node arguments, 
					{self variable: (self asLegalName: soundDuration). 
					self variable: (self asLegalName: soundVolume).
					self literal: false};
		  selector: #playTone:duration:volume:repeat:].	
	super acceptMessageNode: node].

#playSoundFile: -> [:node | 
	self withCurrentMorph: (self morphFor: node receiver name) do: [
		installVariables value: currentMorph.
		node receiver: selfVar;
		  arguments: {RBMessageNode receiver: node arguments first selector: #, arguments: {self literal: '.rso'}. 
						self variable: (self asLegalName: soundVolume). 
						self literal: false};
		  selector: #playFile:volume:repeat:].	
	super acceptMessageNode: node].

#playNote: -> [:node |
	self withCurrentMorph: (self morphFor: node receiver name) do: [
		installVariables value: currentMorph.
		node receiver: selfVar;
		  arguments: {self literal: (SqueakNxtPlayer frequencyFor: node arguments first value ifAbsent: [0]). 
						self variable: (self asLegalName: soundDuration). 
						self variable: (self asLegalName: soundVolume). 
						self literal: false};
		  selector: #playTone:duration:volume:repeat:].	
	super acceptMessageNode: node].

#stopPlaying -> [:node | "For some reason I don't yet understand, if I don't add a Wait(1) the StopSound() doesn't work"
	node arguments: {self literal: 'Wait(1); StopSound();'}; 
		selector: #nxc:; 
		receiver: selfVar.
	super acceptMessageNode: node].
}! !

!NxtPlayerTranslator methodsFor: 'accessing-rules' stamp: 'Richo 3/27/2012 12:22'!
timerRules
| selfVar installVariable firstTick |
selfVar := self variable: 'self'.
firstTick := (#firstTick , self hashString) asSymbol.
installVariable := ["If any of these methods is called I need to install this variable"
	(variables at: currentMorph) at: firstTick put: #CUnsignedLong -> (RBMessageNode receiver: selfVar selector: #currentTick)].
^ {
#resetTimer -> [:node | | newNode | "firstTick := self currentTick."
	installVariable value.
	newNode := RBAssignmentNode 
						variable: (self variable: (self asLegalName: firstTick)) 
						value: (RBMessageNode receiver: selfVar selector: #currentTick).
	node become: (newNode parent: node parent).
	self visitNode: node].

#getCurrentTick -> [:node | "(self currentTick - firstTick)"
	installVariable value.
	node 
		arguments: {self variable: (self asLegalName: firstTick)};
		selector: #-;
		receiver: (RBMessageNode receiver: selfVar selector: #currentTick). 
	self visitNode: node.
]
}! !

!NxtPlayerTranslator methodsFor: 'translating' stamp: 'Richo 3/29/2012 14:28'!
translateMainScript: scriptName of: scriptOwner
	"It translates the script to nxc. It creates a temporary NxtProgram subclass to hold the code which will be translated to nxc. After translation is done, it removes the temporary class.
	Steps:
	* Collect relevant information (motors and sensors composing the robot, slots, accessors and scripts).
	* Compile #setUp method.
	* Compile #main method.
	* Compile the script task.
	* Compile all scripts.
	* Remove temporary class.	
	* Compile temporary class.
	* Translate program to nxc
	* Return nxc program"
	self collectRelevantInformation.
	self
		assert: [self allMorphs includes: scriptOwner]
		description: 'The morph is not connected to the Nxt brick';
		assert: [scriptOwner assuredPlayer class scripts includesKey: scriptName]
		description: 'The script doesn''t belong to the morph'.
	methods := Set new. 
	self
		scriptSelectorsDo: [:each | 
			| node | 
			self visitNode: (node := self currentPlayerClass parseTreeFor: each).
			(variables at: currentMorph)
				at: (self asScriptFlag: each)
				put: #Boolean -> (each = scriptName and: [currentMorph = scriptOwner]).
			(variables at: currentMorph) at: (self asScriptLastTime: each) put: #CUnsignedLong -> 0.
			methods add: node formattedCode].
	self
		withCurrentMorph: scriptOwner
		do: [self compileLoopMethod; compileSetUpMethod].
	self removeNxtProgramSubclass;
		compileNxtProgramSubclass.
	^ self programClass translatedToNxc: #main named: scriptName! !

!NxtPlayerTranslator methodsFor: 'creating parse nodes' stamp: 'Richo 1/27/2011 18:57'!
variable: variableName
	^ RBVariableNode named: variableName! !

!NxtPlayerTranslator methodsFor: 'iterating' stamp: 'Richo 1/11/2011 08:54'!
variablesDo: aBlock 
	"Iterates over the variables dictionary taking care of setting the current morph"
	variables
		keysAndValuesDo: [:morph :vars | self
				withCurrentMorph: morph
				do: [vars keysAndValuesDo: aBlock]] ! !

!NxtPlayerTranslator methodsFor: 'translating' stamp: 'Richo 3/27/2012 11:47'!
withCurrentMorph: aMorph do: aBlock
	"It executes the block using the morph argument as our current morph. It also remembers the old morph, so it's safe to nest this expression.
	The current morph is used whenever we need a legal name for a variable/method name. It defines which morph is currently represented by the 'self' variable"
	| old |
	old := currentMorph.
	currentMorph := aMorph.
	aBlock ensure: [currentMorph := old]! !

NxtProtocol subclass: #NxtCompiledMode
	instanceVariableNames: 'port deviceName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-EtoysCompiler'!

!NxtCompiledMode methodsFor: 'actions' stamp: 'Richo 3/27/2012 10:30'!
beep
	"Temporary we use the direct command protocol just to make 
	a little beep showing the user that we connected succesfully."
	port ifNil: [^ self].
	[DirectCommandProtocol new
		connectOnPort: port;
		beep;
		disconnect]
		on: Error
		do: ["If the direct command protocol couldn't do it, chances are 
			that I won't work. Just in case, disconnect"
			self disconnect]! !

!NxtCompiledMode methodsFor: 'motor actions' stamp: 'Richo 12/23/2011 18:35'!
brakeMotorOnPort: aSmallInteger 
	"Do nothing"! !

!NxtCompiledMode methodsFor: 'connection' stamp: 'Richo 3/28/2012 15:13'!
connectOnPort: aByteSymbol 
	self isConnected ifTrue: [^ self].
	(port := aByteSymbol) = #USB
		ifTrue: [^ self].
	[deviceName := Fantom deviceNameOn: port]
		on: Error
		do: ["If the name search failed, I probably won't work.
			Just in case, disconnect"
			self disconnect]! !

!NxtCompiledMode methodsFor: 'sensor connection' stamp: 'Richo 12/23/2011 18:39'!
correctTypeOf: aSwitchSensor onPort: aSmallInteger 
	"Do nothing"! !

!NxtCompiledMode methodsFor: 'accessing' stamp: 'Richo 3/28/2012 12:36'!
deviceName
	^ deviceName! !

!NxtCompiledMode methodsFor: 'connection' stamp: 'Richo 3/28/2012 12:36'!
disconnect
	port := deviceName := nil! !

!NxtCompiledMode methodsFor: 'motor actions' stamp: 'Richo 12/23/2011 18:35'!
floatMotorOnPort: aSmallInteger 
	"Do nothing"! !

!NxtCompiledMode methodsFor: 'motor actions' stamp: 'Richo 12/23/2011 18:38'!
getPowerOnPort: aSmallInteger 
	"Do nothing"
	^ 0! !

!NxtCompiledMode methodsFor: 'sensor actions' stamp: 'Richo 12/23/2011 18:40'!
getSensorNormalizedValueOnPort: aSmallInteger 
	"Do nothing"
	^ 0! !

!NxtCompiledMode methodsFor: 'sensor actions' stamp: 'Richo 3/27/2012 08:42'!
getSensorRawValueOnPort: aSmallInteger 
	"Do nothing"
	^ 0! !

!NxtCompiledMode methodsFor: 'sensor actions' stamp: 'Richo 3/27/2012 08:42'!
getSensorScaledValueOnPort: aSmallInteger 
	"Do nothing"
	^ 0! !

!NxtCompiledMode methodsFor: 'sensor actions' stamp: 'Richo 12/23/2011 18:40'!
getSensorUltrasonicValueOnPort: aSmallInteger 
	"Do nothing"
	^ 0! !

!NxtCompiledMode methodsFor: 'testing' stamp: 'Richo 12/23/2011 18:25'!
isCompiled
	^ true! !

!NxtCompiledMode methodsFor: 'connection' stamp: 'Richo 3/28/2012 12:35'!
isConnected
	^ port notNil! !

!NxtCompiledMode methodsFor: 'accessing' stamp: 'Richo 12/23/2011 18:21'!
nameSymbol
	^ #compiled! !

!NxtCompiledMode methodsFor: 'actions' stamp: 'Richo 12/23/2011 18:28'!
playSoundFile: aByteString withLoop: aFalse 
	"Do nothing"! !

!NxtCompiledMode methodsFor: 'actions' stamp: 'Richo 12/23/2011 18:25'!
playTone: aSmallInteger duration: aSmallInteger2 
	"Do nothing"! !

!NxtCompiledMode methodsFor: 'motor connections' stamp: 'Richo 12/23/2011 18:35'!
plugMotor: aNxtMotor onPort: aSmallInteger 
	"Do nothing"! !

!NxtCompiledMode methodsFor: 'sensor connection' stamp: 'Richo 12/23/2011 18:35'!
plugSensor: aSwitchSensor onPort: aSmallInteger 
	"Do nothing"! !

!NxtCompiledMode methodsFor: 'motor actions' stamp: 'Richo 12/23/2011 18:41'!
setPower: aSmallInteger onPort: aSmallInteger2 
	"Do nothing"! !

!NxtCompiledMode methodsFor: 'actions' stamp: 'Richo 12/23/2011 18:28'!
startProgram: aByteString 
	"Do nothing"! !

!NxtCompiledMode methodsFor: 'actions' stamp: 'Richo 12/23/2011 18:28'!
stopPlayingSound
	"Do nothing"! !

!NxtCompiledMode methodsFor: 'actions' stamp: 'Richo 12/23/2011 18:28'!
stopProgram
	"Do nothing"! !

!NxtCompiledMode methodsFor: 'motor connections' stamp: 'Richo 12/23/2011 18:35'!
unplugMotorOnPort: aSmallInteger 
	"Do nothing"! !

!NxtCompiledMode methodsFor: 'sensor connection' stamp: 'Richo 12/23/2011 18:35'!
unplugSensorOnPort: aSmallInteger 
	"Do nothing"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
abs
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
address
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
and: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
atRandom
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
buttonPressed: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
currentTick
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
doWhileTrue: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
float: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler' stamp: 'Richo 12/23/2011 18:10'!
getNxtMode
	^ self costume renderedMorph mode nameSymbol! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
ifFalse: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
ifTrue: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler' stamp: 'Richo 8/5/2011 17:19'!
isUsingCompiledMode
	"An error might be signaled if this message is sent to an unplugged device"
	^ [self costume renderedMorph detectMainMorph isUsingCompiledMode] on: Error do: [false]! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
main
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
motorPower: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
motorSpeed: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
negated
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler' stamp: 'Richo 1/12/2011 16:36'!
newScriptorAround: aPhrase
	| scriptEditor |
	Transcript show: 'newScriptorAround:'; cr.
	^ scriptEditor := super newScriptorAround: aPhrase.
	"^ NxtScriptEditorMorph fromScriptEditor: scriptEditor phrase: aPhrase"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
not
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
off: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
or: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
playFile: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
pointer
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
printf: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
random
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
repeat
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler' stamp: 'Richo 1/12/2011 16:37'!
scriptEditorFor: aSelector
	Transcript show: 'scriptEditorFor:'; cr.
	^ super scriptEditorFor: aSelector! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler' stamp: 'Richo 1/10/2011 15:32'!
scriptEditorForNoCng: aSelector
	Transcript show: 'scriptEditorForNoCng:'; cr.
	^ super scriptEditorForNoCng: aSelector.! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
sensorBoolean: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
sensorNormalized: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
sensorRaw: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
sensorScaled: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
sensorType: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
sensorUS: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
sensorValue: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler' stamp: 'Richo 3/26/2012 18:02'!
setNxtMode: aSymbol
	self getNxtMode = aSymbol ifTrue: [^self].
	self costume renderedMorph mode: (aSymbol caseOf: {
		[#direct] -> [DirectCommandProtocol new].
		[#compiled] -> [NxtCompiledMode new].
	} otherwise: [^self]).
	self updateScriptEditors! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
setSensorLight: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
setSensorLowSpeed: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
setSensorSound: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
setSensorTouch: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
setUp
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
sign
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
sqrt
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
system: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
timesRepeat: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler' stamp: 'Richo 3/26/2012 18:01'!
updateScriptEditors
	"Horrible hack below"
	World allMorphsDo: [:morph | | viewer |
		"Update scripts category of all opened viewers"
		((morph isKindOf: StandardViewer)
		and: [morph playerScripted isKindOf: self class officialClass]) 
			ifTrue: [morph updateScriptsCategory].
		"Also update closed viewers"
		((morph isKindOf: ViewerFlapTab) and: [(viewer := morph referent findA: StandardViewer) notNil])
			ifTrue: [viewer updateScriptsCategory].
		"Finally, update all opened script editors"
		((morph isKindOf: ScriptEditorMorph)
		and: [morph playerScripted isKindOf: self class officialClass]) 
			ifTrue: [morph replaceRow1; minWidth: nil]]! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
wait: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
whileTrue
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !

!SqueakNxtPlayer methodsFor: '*squeaknxt-etoyscompiler-nxc commands' stamp: 'Richo 7/29/2011 16:34'!
whileTrue: obj1 
"This method's name is a nxc reserved word. I declared it just to prevent the user from creating a script with this same name"! !
